@@ -86,6 +86,21 @@ default_build_command(const char **cmd, const char ***argv,
 
 static void fcgid_add_cgi_vars(request_rec * r)
 {
+	apr_array_header_t *passheaders = get_pass_headers(r);
+
+	if (passheaders != NULL) {
+		const char **hdr = (const char **) passheaders->elts;
+		int hdrcnt = passheaders->nelts;
+		int i;
+
+		for (i = 0; i < hdrcnt; i++, ++hdr) {
+			const char *val = apr_table_get(r->headers_in, *hdr);
+
+			if (val)
+				apr_table_setn(r->subprocess_env, *hdr, val);
+		}
+	}
+
 	/* Work around cgi.fix_pathinfo = 1 in php.ini */
 	if (g_php_fix_pathinfo_enable) {
 		char *merge_path;
@@ -224,6 +239,7 @@ static int mod_fcgid_authenticator(request_rec * r)
 	/* Add some environment variables */
 	ap_add_common_vars(r);
 	ap_add_cgi_vars(r);
+	fcgid_add_cgi_vars(r);
 	apr_table_setn(r->subprocess_env, "REMOTE_PASSWD", password);
 	apr_table_setn(r->subprocess_env, "FCGI_APACHE_ROLE", "AUTHENTICATOR");
 
@@ -304,6 +320,7 @@ static int mod_fcgid_authorizer(request_rec * r)
 	/* Add some environment variables */
 	ap_add_common_vars(r);
 	ap_add_cgi_vars(r);
+	fcgid_add_cgi_vars(r);
 	apr_table_setn(r->subprocess_env, "FCGI_APACHE_ROLE", "AUTHORIZER");
 
 	/* Remove some environment variables */
@@ -383,6 +400,7 @@ static int mod_fcgid_check_access(request_rec * r)
 	/* Add some environment variables */
 	ap_add_common_vars(r);
 	ap_add_cgi_vars(r);
+	fcgid_add_cgi_vars(r);
 	apr_table_setn(r->subprocess_env, "FCGI_APACHE_ROLE",
 				   "ACCESS_CHECKER");
 
@@ -540,6 +558,9 @@ static const command_rec fcgid_cmds[] = {
 				  "Spawn score up limit"),
 	AP_INIT_TAKE1("SpawnScore", set_spawn_score, NULL, RSRC_CONF,
 				  "Score of spawn"),
+	AP_INIT_TAKE1("TimeScore", set_time_score, NULL,
+				  RSRC_CONF,
+				  "Score of passage of time (in seconds)"),
 	AP_INIT_TAKE1("TerminationScore", set_termination_score, NULL,
 				  RSRC_CONF,
 				  "Score of termination"),
@@ -561,8 +582,15 @@ static const command_rec fcgid_cmds[] = {
 				  "Connect timeout to fastcgi server"),
 	AP_INIT_TAKE1("IPCCommTimeout", set_ipc_comm_timeout, NULL, RSRC_CONF,
 				  "Communication timeout to fastcgi server"),
+	AP_INIT_TAKE1("MaxRequestLen", set_max_request_len, NULL, RSRC_CONF,
+				  "Max HTTP request length in byte"),
+	AP_INIT_TAKE1("MaxRequestInMem", set_max_mem_request_len, NULL,
+				  RSRC_CONF,
+				  "The part of HTTP request which greater than this limit will swap to disk"),
 	AP_INIT_TAKE12("DefaultInitEnv", add_default_env_vars, NULL, RSRC_CONF,
 				   "an environment variable name and optional value to pass to FastCGI."),
+	AP_INIT_TAKE1("PassHeader", add_pass_headers, NULL, RSRC_CONF,
+				  "Header name which will be passed to FastCGI as environment variable."),
 	AP_INIT_TAKE12("FCGIWrapper", set_wrapper_config, NULL,
 				   RSRC_CONF | ACCESS_CONF | OR_FILEINFO,
 				   "The CGI wrapper setting"),
